"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DataLoader = require("dataloader");
const lodash_1 = require("lodash");
const MAX_BATCH_SIZE = 100;
function createDataLoader(options) {
    // Keep a local reference to the client so that it cannot be
    // changed by the caller after creating the data loader
    const client = options.client;
    const loader = new DataLoader(async (itemsToGet) => {
        // DynamoDB does not allow duplicate keys in the batchGet requests
        // do de-duplicate the keys before building the request
        const uniqueKeys = lodash_1.uniqWith(itemsToGet, lodash_1.isEqual);
        // Groups the items together by table into the request format that
        // DynamoDB expects
        const itemsByTable = lodash_1.groupBy(uniqueKeys, 'table');
        const requestItems = lodash_1.mapValues(itemsByTable, (itemsToGet) => {
            return { Keys: lodash_1.map(itemsToGet, 'key') };
        });
        // Perform the batch lookup
        const results = await client
            .batchGetItem({ RequestItems: requestItems })
            .promise();
        // Map the results from DyanmoDB into an array of results in the same
        // order as the requested items
        return itemsToGet.map((itemToGet) => {
            if (results.Responses) {
                const tableResults = results.Responses[itemToGet.table];
                // Check if the item was found
                if (tableResults) {
                    const itemResult = lodash_1.find(tableResults, itemToGet.key);
                    if (itemResult)
                        return itemResult;
                }
            }
            // If there are any unprocessed keys, treat them as failures that can
            // be retried
            if (results.UnprocessedKeys && !lodash_1.isEmpty(results.UnprocessedKeys)) {
                const tableResults = results.UnprocessedKeys[itemToGet.table];
                if (tableResults) {
                    const itemResult = lodash_1.find(tableResults.Keys, itemToGet.key);
                    if (itemResult)
                        return new Error('The item was not processed');
                }
            }
            return null;
        });
    }, { maxBatchSize: MAX_BATCH_SIZE });
    return loader;
}
exports.createDataLoader = createDataLoader;
